//
// Created by trokay on 13.06.23.
//

#include <termios.h>
#include <syslog.h>

#include "protocol.h"
#include "uart.h"
#include "headers.h"

int init_uart(struct uart_t *uart, const char *name, int baudrate) {
  syslog(LOG_INFO,"open uart %s\n", name);
  uart->fd = open(name, O_RDWR | O_NOCTTY | O_NDELAY);
  if (uart->fd == -1) {
    syslog(LOG_EMERG,"Failed to open UART device");
    return -1;
  }

  syslog(LOG_DEBUG,"Configure UART settings start");
  struct termios uart_settings;
  memset(&uart_settings, 0, sizeof(uart_settings));
  uart_settings.c_cflag &= ~PARENB; // Disabling parity
  uart_settings.c_cflag &= ~CSTOPB; // Clear stop field, only one stop bit used in communication (most common)
  uart_settings.c_cflag &= ~CSIZE; // Clear all bits that set the data size
  uart_settings.c_iflag &= ~IXOFF;
  uart_settings.c_iflag &= ~IXON;
  uart_settings.c_iflag |= IGNCR;
  uart_settings.c_iflag |= IGNBRK;
  uart_settings.c_cflag |= CS8; // 8 bits per byte
  uart_settings.c_cflag &= ~CRTSCTS; // Disable RTS/CTS hardware flow control
  uart_settings.c_cflag |= CREAD | CLOCAL; // Turn on READ & ignore ctrl lines (CLOCAL = 1)
  //uart_settings.c_iflag = IGNPAR;
  uart_settings.c_oflag = 0;
  uart_settings.c_lflag = 0;
  uart_settings.c_iflag = 0;
  uart_settings.c_cc[VTIME] = 10;    // Wait for up to 1s (10 deciseconds), returning as soon as any data is received.
  uart_settings.c_cc[VMIN] = 0;

    //uart_settings.c_oflag &= ~OPOST; // Prevent special interpretation of output bytes (e.g. newline chars)
    //uart_settings.c_oflag &= ~ONLCR; // Prevent conversion of newline to carriage return/line feed

  tcflush(uart->fd, TCIFLUSH);
  cfsetspeed(&uart_settings, baudrate);
  tcsetattr(uart->fd, TCSANOW, &uart_settings);
  syslog(LOG_DEBUG,"Configure UART settings finish");
  return 0;
}

//TODO написать функцию
int write_uart(struct uart_t *uart) {
    close(uart->fd);
    return 0;
}

//TODO использовать функцию
int close_uart(struct uart_t *uart) {
    close(uart->fd);
    return 0;
}

int read_uart_circle(struct uart_t *uart) {

  syslog(LOG_DEBUG,"get data");
  int tmp = 0;
  if (uart->tail < uart->head) {
    tmp = uart->head - uart->tail;
  } else {
    tmp = BUFFER_SIZE - uart->tail;
  }
  ssize_t num_bytes = read(uart->fd, &(uart->buffer[uart->tail]), tmp);
  syslog(LOG_DEBUG, "get %ld bytes data", num_bytes);
  uart->tail += num_bytes;
  if (uart->tail == BUFFER_SIZE) {
    uart->tail = 0;
  }

  do {
    if (uart->flag & 0x80) {
      if (uart->buffer[uart->head] == DELIMITER_FLAG) {
	uart->flag |= 0x81;
	uart->message[uart->size++] = DELIMITER_FLAG;
	//callback
	if (uart->size > 2) {
	  uart->invoke(uart);
	}
	uart->size = 0;
	uart->flag = 0x80;
	uart->message[uart->size++] = DELIMITER_FLAG;
      } else {
	uart->message[uart->size++] = uart->buffer[uart->head];
      }
    } else {
      if (uart->buffer[uart->head] == DELIMITER_FLAG) {
	uart->flag |= 0x80;
	uart->message[uart->size++] = DELIMITER_FLAG;
      }
    }
    uart->head++;
    if (uart->head == BUFFER_SIZE) {
      uart->head = 0;
    }
  } while (uart->head != uart->tail);

  return 0;
}
